//
//	docs.js
//
//	Parse markdown and generate navigation
//
var _ = require('./help.js'),
		fs = require('fs'),
		marked = require('marked'),
		prism = require('../node_modules/prismjs/components/prism-core.js');

		require('../node_modules/prismjs/components/prism-bash.js');
    require('../node_modules/prismjs/components/prism-markup.js');
    require('../node_modules/prismjs/components/prism-css.js');
    require('../node_modules/prismjs/components/prism-scss.js');
    require('../node_modules/prismjs/components/prism-clike.js');
    require('../node_modules/prismjs/components/prism-javascript.js');

module.exports = (function(){
	

	/**
	 *	Takes a file path
	 *
	 *	Gets markdown contents
	 *	Parses to HTML
	 *	Combs HTML for h1, h2, and h3 tags
	 *	Builds navigation structure
	 *	Builds content structure
	 *	
	 *	Returns data structure representing docs
	 */

	var data = {};


	var titleTeaser = /(<h1[^]+?<\/h1>)([^]+?)(?:<h2|%%END%%)/g;
	var sections = /(<h2[^]+?<\/h2>)([^]+?)(?:%%H2%%|%%END%%)/g;
	var codeBlocks = /(<pre>[^]*?<code class="lang-(\w+)">([^]+?)<\/code>)/g;
	var mdcodeBlock = /```(\w+)([^]+?)```/g;
	var mdcodeInline = /[^`]`([^`]+?)`[^`]/g;
	var htmlcodeInline = /(<code>([^]+?)<\/code>)/g;
	var htmlcodeBlock = /(<pre>[^]*?<code class="lang-(\w+)">([^]+?)<\/code>)/g;

	function highlightCode(content) {

		var output = content;

		var mdCode = _.find(output, mdcodeInline, true, 1)
				.map(function(inline) {
					return {type: 'inline', code: inline};
				}).concat(_.find(output, mdcodeBlock, true, 2)
					.map(function(block) {
						return {type: 'block', lang: block[1], code: block[2]}
					})
				);

		output = marked(output);

		var htmlCode = _.find(output, htmlcodeInline, true, 2)
				.map(function(inline) {
					return { type: 'inline', full: inline[1], inner: inline[2] };
				}).concat(_.find(output, htmlcodeBlock, true, 3)
					.map(function(block) {
						return {
							type: 'block',
							full: block[1],
							lang: block[2],
							inner: block[3]
						}
					})
				);

		htmlCode.forEach(function(codeObj, i) {
			if (codeObj.type === 'inline') {

				var hl = prism.highlight(mdCode[i].code, prism.languages.markup);
				var newCode = '<code class="sm-docs-codeinline language-markup">' + hl + '</code>';
				output = output.replace(codeObj.full, newCode);

			} else {

				var hl = prism.highlight(mdCode[i].code, prism.languages[mdCode[i].lang]);
				var newCode = '<code class="sm-docs-codeblock"><pre class="sm-docs-codepre language-';
				newCode += mdCode[i].lang + '">' + hl + '</pre></code>';
				output = output.replace(codeObj.full, newCode);

			}

		})

		// fs.writeFileSync('tests/testdocs.html', output);

		return output;

	}

	function makeDocs(path) {

		// Setup
		var content = highlightCode(_.contents(path) + '\n%%END%%');

		var splash = _.find(content, titleTeaser, true, 2).map(function(part) {
			var id = _.find(part[1], /<h1 id="([^]+?)"/g, false);
			var name = _.find(part[1], />([^]+?)<\/h1>/g, false); 
			var output = '<div class="sm-docs-section-wrap isActive" data-content="'+id+'">';
			output += part[1].trim();
			output += part[2].replace(/%%H2%%|%%H3%%|%%END%%/g, '').trim() + '</div>';
			return {
				link: '<span class="sm-docs-nav-title" rel="'+id+'">'+name+'</span>',
				content: output
			}
		});

		
		// Mark h2 boundaries
		_.find(content, /(<h2[^]+?<\/h2>)/g).forEach(function(h2) {
			content = content.replace(h2, '%%H2%%\n' + h2);
		});

		// Mark h3 boundaries
		_.find(content, /(<h3[^]+?<\/h3>)/g).forEach(function(h3) {
			content = content.replace(h3, '%%H3%%\n' + h3);
		});

		// Build structures
		var structures = _.find(content, sections, true, 2)
			.map(function(section) {

				var name = _.find(section[1], />([^]+?)<\/h2>/g, false);
				var id = _.find(section[1], /<h2 id="([^]+?)"/g, false);
				var content = section[1] + section[2].replace(/%%H2%%|%%H3%%|%%END%%/g, '').trim();

				return {
					link: '<li class="sm-docs-nav-section"><a class="sm-docs-nav-section-text" href="#'+id+'">'+name+'</a>',
					id: id,
					content: '<div class="sm-docs-section-wrap" data-content="'+id+'">'+content+'</div>'
				}

			})
			.map(function(section) {

				section.subheadings = _.find(section.content, /(<h3[^]+?<\/h3>)/g);

				if (section.subheadings.length > 0) {
					section.subheadings = '<ul class="sm-docs-nav-sublist">' + section.subheadings.map(function(sub) {

						var name = _.find(sub, />([^]+?)<\/h3>/g, false);
						var id = _.find(sub, /<h3 id="([^]+?)"/g, false);

						section.content = section.content.replace('<h3 id="'+id+'">', '<h3 id="'+section.id + '+' + id+'">');

						return '<li class="sm-docs-nav-sub"><a class="sm-docs-nav-sublink" href="#'+section.id + '+' + id+'">'+name+'</a></li>';
					
					}).join('\n') + '</ul>';
				} else {
					section.subheadings = null;
				}

				return section;

			});


		var output = {
			splash: splash[0],
			structure: structures
		}

		return output;

	}


	return makeDocs;


})()