//
//  interpret.js
//  ------------
//
//  Interprets parsed templating language and returns HTML
//
var _ = require('./help.js'),
    fs = require('fs'),
    run = require('child_process'),
    gill = require('./highlight.js'),
    beautify = require('js-beautify').html,
    prism = require('../node_modules/prismjs/components/prism-core.js');

    require('../node_modules/prismjs/components/prism-clike.js');
    require('../node_modules/prismjs/components/prism-markup.js');
    require('../node_modules/prismjs/components/prism-css.js');
    require('../node_modules/prismjs/components/prism-scss.js');

module.exports = (function() {


  var MODELS, OUTPUT, linkNum = 0;


  function interpretHTMLFragment(node) {
    return node.value;
  }


  function interpretStringParam(node, args) {
    if (args && args.hasOwnProperty(node.paramName)) {
      if (typeof(args[node.paramName]) === 'string') {
        return args[node.paramName];
      } else {
        return interpretArgs(args[node.paramName]);
      }
    } else {
      return node.arg;
    }
  }


  function interpretModelParam(node, args) {
    if (args && args.hasOwnProperty(node.paramName)) {
      return args[node.paramName];
    } else {
      if (node.modelName) {
        return interpretNodes(MODELS[node.modelName].parsed, interpretArgs(node.args))
      } else {
        return '';
      }
    }
  }


  function interpretArgs(args) {
    var interpretedArgs = _.copy(args);
    for (var arg in interpretedArgs) {
      interpretedArgs[arg] = interpretNodes(interpretedArgs[arg]);
    }
    return interpretedArgs;
  }


  function interpretModelRef(node) {
    return interpretNodes(MODELS[node.name].parsed, interpretArgs(node.args));
  }


  function interpretNodes(nodes, args) {

    var markup = '';

    nodes.forEach(function(node) {

      if (node.type === 'HTMLFragment') {
        markup += interpretHTMLFragment(node);
      } else

      if (node.type === 'StringParam') {
        markup += interpretStringParam(node, args);
      } else

      if (node.type === 'ModelRef') {
        markup += interpretModelRef(node);
      } else

      if (node.type === 'ModelParam') {
        markup += interpretModelParam(node, args);
      }

    })
    
    return markup;

  }


  function getRefs(nodes, args, refs) {

    var refs = refs || [],
        args = args || [];

    nodes.forEach(function(node) {

      if (node.type === 'ModelRef') {
        if (!refs.hasOwnProperty(node.name)) {
          refs.push(node.name);
          refs = getRefs(MODELS[node.name].parsed, node.args, refs);
        }
      }

      if (node.type === 'StringParam') {
        if (args && args.hasOwnProperty(node.paramName)) {
          refs = getRefs(args[node.paramName], null, refs);
        }
      }

      if (node.type === 'ModelParam') {
        if (args && args.hasOwnProperty(node.paramName)) {
          refs = getRefs(args[node.paramName], null, refs);
        } else {
          if (!refs.hasOwnProperty(node.modelName)) {
            if (node.modelName) {
              refs.push(node.modelName);
              refs = getRefs(MODELS[node.modelName].parsed, node.args, refs);
            }
          }
        }

      } // end node loop

    })

    return refs;

  }

  function buildNestedCallsDeep(model, logged) {

    var callList = logged || [];
    var calls = _.removeDuplicates(model.includes.concat(model.references));

    if (calls.length) {

      calls.forEach(function(name) {
        callList = buildNestedCallsDeep(MODELS[name], callList);
      })

      calls.forEach(function(name) {
        if (callList.indexOf(name) < 0) {
          callList.push(name);
        }
      })

    }

    return callList;

  }


  function buildModelCss(model) {
    
    var fullSass = '';
    var fullCss = '';

    model.calls.forEach(function(name) {
      fullSass += MODELS[name].sass + '\n';
      fullCss += MODELS[name].css + '\n';
    })

    fullSass += model.sass;
    fullCss += model.css;

    return { css: fullCss, sass: fullSass };
  }

  function removeEmptyStrings(str) {
    return str !== '';
  }

  function buildPaths(path) {

    var Path = _.truePath(path), pathStr;

    var dirs = Path.split(_.slash).filter(removeEmptyStrings);
    pathStr = _.os === 'win32' ? '' : '/';
    
    dirs.forEach(function(dir, i) {
      pathStr += dir;
      if (i !== dirs.length - 1) {
        pathStr += _.slash;
      }
      
      _.ifExists(pathStr, null, function(dir, e) {
        if (!dir.match(/(\.[^\/\\]+)$/)) {
          run.execSync('mkdir ' + dir);
        }
      })
      
    })

  }

  function buildModelJs(model, jsSource, pageOutput, allJS, logged) {
    
    var fullJs = allJS || [];
    var logged = logged || [];
    
    if (model.calls) {
      model.calls.forEach(function(ref) {
        fullJs.push(buildModelJs(MODELS[ref], jsSource, pageOutput, fullJs, logged));
      })
    }
    
    model.js.forEach(function(js) {
      if (logged.indexOf(js) < 0) {
        var file = _.endSlash(process.cwd()) + jsSource + js;
        var dest = _.endSlash(process.cwd()) + pageOutput + 'js/' + js;
        
        buildPaths(dest);
        logged.push(js);

        fs.writeFileSync(dest, _.contents(file));
        fullJs.push('<script type="text/javascript" src="../js/' + js + '"></script>');

      }
      
    })

    fullJs = _.removeDuplicates(fullJs).join('');

    return fullJs;
  }

  function convertProjectRoot(interpretation, projectRoot) {
    var calls = interpretation.match(/\{%.+?%\}/g);
    var asset;
    var link;
    var command;

    if (calls) {

      if (!projectRoot) {
        _.err('Root token used but no root supplied. Assign valid path to project_root in options.');
      }

      calls.forEach(function(call) {

        var cleanCall = call.replace(/\{%|%\}/g, '').trim();
        var assetAbsolutePath = _.endSlash(process.cwd()) + _.endSlash(projectRoot) + _.truePath(cleanCall);

        asset = 'asset' + linkNum++;
        link = 'links' + _.slash + asset;

        run.execSync('ln -s ' + assetAbsolutePath + ' ' + _.endSlash(process.cwd()) + _.endSlash(OUTPUT) + link);
        
        interpretation = interpretation.replace(call, '../links/' + asset);
      })

    }
    return interpretation;
  }

  function convertModelRefs(refs) {
    return refs.map(function(ref) {
      var r = MODELS[ref];
      return r.section+'|'+r.group+'|'+r.display;
    })
  }

  function wrapStyleDependencies(highlighted, name) {

    if (MODELS[name].references.length > 0 || MODELS[name].includes.length > 0) {
      var regex = new RegExp('<span.+?\\/\\*{2}\\s*\\*\\s*model: ' + name);
      var found = highlighted.match(regex);
      var index;

      if (found) {
        index = highlighted.indexOf(found[0]);
      }
      if (index > 0) {
        var dependencies = highlighted.substring(0, index - 1);
        var wrappedDependents = dependencies
          .replace(/(<span\sclass="token\scomment".+?\/\*\*[^]+?model:\s(.+))/g, '</span>]MODEL~$2~[$1')
          .replace(/\]MODEL~(.+)~\[/g, '<span class="sm-style-dependent-btn">$1</span><span class="sm-style-dependent-wrap">')
          .replace(/^(<\/span>)/, '');

        var wrapped = '<span class="sm-style-dependencies"><span class="sm-style-dependencies-btn">Show Dependencies</span><span class="sm-style-dependencies-container">'+wrappedDependents+'</span><span class="sm-style-dependent-fill"></span></span>';
        return highlighted.replace(dependencies, wrapped);
      } else {
        return highlighted;
      }
    } else {
      return highlighted;
    }
  }


  function interpretParsed(data, root) {
    return beautify(convertProjectRoot(interpretNodes(data), root), {indent_size: 2});
  }


  function interpretModels(models, pageOutput, jsSource, projectRoot, sassSource) {

    MODELS = models || _.err('Interpretation requires models object');
    OUTPUT = pageOutput,
    output = [],
    dataOutput = {};

    //
    //  FIRST PASS
    //
    for (model in MODELS) {
      var M = MODELS[model];

      M.references = getRefs(M.parsed).reverse();
      M.interpretation = interpretParsed(M.parsed, projectRoot);
      
      if (M.parsedFull) {
        M.interpretationFull = interpretParsed(M.parsedFull, projectRoot);
      }

      M.file = M.file.replace(sassSource, '');
      M.sass = '/**\n * model: ' + M.name + '\n * file: ' + M.file + '\n */\n' + M.sass + '\n';
      M.css = '/**\n * model: ' + M.name + '\n * file: ' + M.file + '\n */\n' + M.css + '\n';

      output.push(M);
    }

    var frameTemplate = _.contents(__dirname + '/../templates/model_frame.html');


    //
    //  SECOND PASS
    //
    for (model in MODELS) {
      var M = MODELS[model];

      M.calls = buildNestedCallsDeep(M);

      var styles = buildModelCss(M);
      
      M.fullcss = styles.css;
      M.fullsass = styles.sass;
      M.highlightedTemplate = gill(M.template);
      M.highlightedSass = wrapStyleDependencies(prism.highlight(M.fullsass, prism.languages.scss), M.name);
      M.highlightedCss = wrapStyleDependencies(prism.highlight(M.fullcss, prism.languages.css), M.name);
      M.highlightedInterpretation = prism.highlight(M.interpretation, prism.languages.markup);

      var iFrameContents = frameTemplate.replace('{{modelCss}}', M.fullcss);

      if (M.interpretationFull) {
        iFrameContents = iFrameContents.replace('{{modelInterpretation}}', M.interpretationFull);
      } else {
        iFrameContents = iFrameContents.replace('{{modelInterpretation}}', M.interpretation);
      }

      var allTheJs = buildModelJs(M, jsSource, pageOutput);

      if (allTheJs) {
        iFrameContents = iFrameContents
          .replace('{{jsDependencies}}', '<script id="js-dependencies" type="text/javascript" src="../js/app/dependencies.js"></script>')
          .replace('{{modelJs}}', allTheJs);
      } else {
        iFrameContents = iFrameContents.replace('{{jsDependencies}}','').replace('{{modelJs}}','');
      }

      fs.writeFileSync(pageOutput + 'models/' + M.name + '.html', iFrameContents);
    }

    //
    //  CLEANUP
    //
    for (model in MODELS) {
      var M = MODELS[model];
      M.references = convertModelRefs(M.references);
      M.includes = convertModelRefs(M.includes);
      dataOutput[M.name] = _.copy(M);

      delete M.css;
      delete M.fullcss;
      delete M.fullsass;
      delete M.parsed;
      delete M.parsedFull;
      delete M.interpretation;
      delete M.sass;
      delete M.template;
      delete M.templateFull;
      delete M.templateUntouched;

    }

    fs.writeFileSync(__dirname + '/../tests/data.json', JSON.stringify(dataOutput, null, 2));

    return output;

  }


  return interpretModels;

})();