//
//  parse.js
//  --------
//
//  Parse model markup block into abstrat syntax tree and return object
//
//  Grammar
//
//  ModelBody     ::=   ModelPart*
//
//  ModelPart     ::=   TextParam | ModelParam | ModelRef | HTMLFragment
//
//  StringParam   ::=   '{+' NAME ':' STRING '+}'
//
//  ModelParam    ::=   '{{+' NAME ':' NAME ModelArg* '+}}'
//
//  ModelArg      ::=   '[[' NAME ']]' (ModelRef | HTMLFragment)*
//
//  ModelRef      ::=   '{{' NAME ModelArg* '}}'
//  
//  HTMLFragment  ::=   HTML | '`' HTML '`'
//
//  NAME          ::=   /[\w\ \-]+/
//
//  STRING        ::=   /[^]*?(?:\+\})/
//

var _ = require('./help.js');


module.exports = (function() {

  // identifies names in templating laguage
  var regExpName = /[\w\ \-]+/;

  // Returns an HTMLFragment data structure with type and value
  // context (object) - contains full string with current index
  // endExp (RegExp) - regular expression for possible language symbols
  function parseHTMLFragment(context, endExp, sensitive) {

    var rest = context.input.substring(context.index),
        nextIndex = rest.search(endExp),
        sensitive = sensitive || false;

    var inTag = rest.match(/^[^<>]+?>/);
    if (inTag) {
      sensitive = true;
    }

    if (nextIndex < 0) {
      context.index = context.input.length;
      rest = sensitive ? rest : rest.trim();
      return {
        type: 'HTMLFragment',
        value: rest
      }
    } else {
      context.index += nextIndex;
      rest = sensitive ? rest.substring(0, nextIndex) : rest.substring(0, nextIndex).trim();
      return {
        type: 'HTMLFragment',
        value: rest
      }
    }

  }

  // Consumes and returns the next found of pattern in context.input
  // Side effect: updates context.index
  function readToken(context, pattern) {

    var rest = context.input.substring(context.index),
        matched = (rest).match(pattern);

    if (matched) {
      context.index += matched[0].length;
      return matched[0];
    }
    else {
      throw "Expected " + pattern;
    }
  }


  function parseModelArg(context) {

    readToken(context, /\[\[/);
    var name = readToken(context, regExpName).trim();
    readToken(context, /\]\]/);
    var sensitive = readToken(context, /(>>)*/) === '>>';
    var argArr = [];

    while (true) {

      if (context.index >= context.input.length) {
        throw "Just, just what?"
      }

      if (!sensitive) readToken(context, /\s*/);

      var rest = context.input.substring(context.index);

      if (sensitive) {
        argArr.push(parseHTMLFragment(context, /<</, sensitive));
        readToken(context, /<<\s*/);
        rest = context.input.substring(context.index);
      }

      if (rest.startsWith('+}}') || rest.startsWith('}}') || rest.startsWith('[[')) {
        break;
      } else if (rest.startsWith('{{')) {
        argArr.push(parseModelRef(context));
      } else {
        argArr.push(parseHTMLFragment(context, /\{\{|\[\[|\+\}\}|\}\}/));
      }

    }

    return {name: name, body: argArr};

  }


  function parseModelRef(context) {

    readToken(context, /\{\{/);
    var name = readToken(context, /[\w\s\-]+/).trim();
    var args = {};
    while (context.input.substring(context.index).startsWith('[[')) {
      var argObj = parseModelArg(context);
      args[argObj.name] = argObj.body;
    }
    readToken(context, /\}\}\s*/);
    return {type: 'ModelRef', name: name, args: args};

  }


  function parseStringParam(context) {

    readToken(context, /\{\+\s*/);
    var paramName = readToken(context, regExpName).trim();
    var paramDivider = readToken(context, /:*/);
    var paramValue = readToken(context, /[^]*?(?:\+\})/).replace(/(?:\s*\+\})/, '');

    return {type: 'StringParam', paramName: paramName, arg: paramValue};

  }
  

  function parseModelParam(context) {

    readToken(context, /\{\{\+\s*/);
    var paramName = readToken(context, regExpName).trim();
    var hasDefault = readToken(context, /(\s*:\s*)*/);
    var modelName = readToken(context, /([\w\s\-]+)*/).trim();
    var args = {};
    if (hasDefault !== '') {
      while (context.input.substring(context.index).startsWith('[[')) {
        var argObj = parseModelArg(context);
        args[argObj.name] = argObj.body;
      }
    }
    readToken(context, /\s*\+\}\}\s*/);
    return {type: 'ModelParam', modelName: modelName, paramName: paramName, args: args};

  }


  function parseModelPart(context) {

    var htmlFrag = parseHTMLFragment(context, /\{\{\+|\{\{|\{\+/);

    if (htmlFrag.value) {
      return htmlFrag;

    } else {

      var rest = context.input.substring(context.index),
          matched = rest.match(/\{\{\+|\{\{|\{\+/),
          nextToken = matched ? matched[0] : null;

      if (nextToken === '{{+') {
        return parseModelParam(context);
      } else if (nextToken === '{{') {
        return parseModelRef(context);
      } else if (nextToken === '{+') {
        return parseStringParam(context);
      } else {
        throw "Invalid string: expected {{, {{+, or {+"
      }

    }
  }


  function parseModelBody(context) {
    var bodyArr = [];
    while (context.index < context.input.length) {
      bodyArr.push(parseModelPart(context))
    }
    return bodyArr;
  }


  // Tests

  // _.cl("\nTest: parseHTMLFragment");
  // _.cl(parseHTMLFragment({input: '<div>\n\tfoo\n</div>\n{{ ref }}', index:0}, /\{\{|\{\{\+|\{\+/));

  // _.cl('\nTest: readToken()');
  // _.cl(readToken({input: '<div>foo</div>{{ref}}', index:14}, /\{\{/));

  // _.cl('\nTest: parseModelArg');
  // _.cl(parseModelArg({input: '{{ Foo\n  [[ bar ]] {{ Baz }}\n}}', index:8}));

  // _.cl('\nTest: parseModelRef()');
  // _.cl(parseModelRef({input: '{{ ref \n\t[[ foo ]] Bar\n}}', index: 0}));

  // _.cl('\nTest: parseStringParam');
  // _.cl(_.prettyJson(parseStringParam({input: '<div{+ attrs: class="foo" +}></div>', index: 4})));

  // _.cl('\nTest: parseModelParam');
  // _.cl(_.prettyJson(parseModelParam({input: '{{+ param : Ref +}}', index: 0})));

  // _.cl("\nTest: parseModelPart");
  // _.cl(parseModelPart({input: '<div>foo</div>{{ref}}', index:0}));

  // _.cl("\nTest: parseModelBody");
  // var testStr = '<div{+attrs+}>{{+ fiz:FOO\n  [[bar]] {{ BAZ\n    [[boo]]` class  `\n    [[fiz]] bleublah }}\n+}}</div>';
  // _.cl(testStr);
  // _.cl(_.prettyJson(parseModelBody({input: testStr, index: 0})));


  return function(templateContent) {
    return parseModelBody({input:templateContent, index:0});
  }


})()

